<?php

namespace Drupal\views_kanban\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\user\Entity\User;
use Drupal\views\Views;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * Class KanbanController.
 */
class KanbanController extends ControllerBase {

  /**
   * @param string $view_id
   * @param string $display_id
   * @param int $entity_id
   * @param string $state_value
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function updateState($view_id, $display_id, $entity_id = 0, $state_value = '') {
    $message = NULL;
    $data = [
      'success' => FALSE,
      'message' => $message,
    ];
    if (!$state_value && !is_numeric($entity_id)) {
      return new JsonResponse($data);
    }
    $view = Views::getView($view_id);
    $entity_type = $view->getHandler($display_id, 'filter', 'type')['entity_type'];
    $style_plugin = $view->display_handler->getPlugin('style');
    $status_field = $style_plugin->options["status_field"];
    $entity = \Drupal::entityTypeManager()
      ->getStorage($entity_type)
      ->load($entity_id);
    $message = $this->getHistoryMessage($entity, $status_field, $state_value);
    //save new status
    $entity->set($status_field, $state_value);

    //save history
    if (!empty($historyFieldName = $style_plugin->options["history_field"])) {
      $historyField = $entity->get($style_plugin->options["history_field"]);
      $historyType = $historyField->getFieldDefinition()->getType();
      $historyValue = \Drupal::service('date.formatter')
          ->format(strtotime('now')) . ' ' . $message;
      if ($historyType == 'double_field') {
        $historyValue = [
          'first' => date('Y-m-d\TH:i:s'),
          'second' => $message,
        ];
      }
      $entity->$historyFieldName->appendItem($historyValue);
    }

    $entity->save();

    //send email to assignor
    if (!empty($style_plugin->options["send_email"])) {
      $assignValues[$entity->getOwnerID()] = [$entity->getOwnerID()];
      if (!empty($style_plugin->options["assign_field"])) {
        $assignors = $entity->get($style_plugin->options["assign_field"])
          ->getValue();
        foreach ($assignors as $assignor) {
          $assignValues[$assignor['target_id']] = $assignor['target_id'];
        }
      }
      $mailManager = \Drupal::service('plugin.manager.mail');
      $module = 'kanban';
      $key = $view_id.'-'.$view->current_display;
      foreach ($assignValues as $uid) {
        $assignor = User::load($entity->getOwnerID());
        $to = $assignor->getEmail();
        $params['message'] = $message;
        $params['title'] = t("Change status") . " - " . $entity->label();
        $langcode = \Drupal::currentUser()->getPreferredLangcode();
        $send = TRUE;
        $result = $mailManager->mail($module, $key, $to, $langcode, $params, NULL, $send);
      }
    }
    $data = [
      'success' => TRUE,
      'message' => $message,
    ];
    return new JsonResponse($data);
  }

  protected function getHistoryMessage($entity, $status_field, $newStatus) {
    $statusFieldDefinition = $entity->get($status_field)->getFieldDefinition();
    $statusFieldValues = $statusFieldDefinition->getSettings();

    if (!empty($statusFieldValues["allowed_values"])) {
      $statusList = $statusFieldValues["allowed_values"];
    }
    if (!empty($statusFieldValues["target_type"])) {
      $vid = current($statusFieldValues["handler_settings"]["target_bundles"]);
      $loadTermStatus = \Drupal::entityTypeManager()
        ->getStorage('taxonomy_term')
        ->loadTree($vid);
      foreach ($loadTermStatus as $term) {
        $statusList[$term->tid] = $term->name;
      }
    }
    $currentStatus = $entity->get($status_field)->getString();
    if(is_array($currentStatus)){
      $currentStatus = current($currentStatus);
    }
    if (!empty($statusList[$currentStatus])) {
      $currentStatus = $statusList[$currentStatus];
    }
    if (!empty($statusList[$newStatus])) {
      $newStatus = $statusList[$newStatus];
    }
    $account = \Drupal::currentUser();
    return t('@user change from @old to @new', [
        '@user' => $account->getDisplayName(),
        '@old' => $currentStatus,
        '@new' => $newStatus,
      ]
    );
  }

}
